﻿using System;
using System.Collections.Generic;
using System.Drawing;

using MindFusion.Diagramming;
using MindFusion.Drawing;


namespace FreehandDraw
{
	class FreeLine : DiagramNode
	{
		public FreeLine()
		{
			points = new List<PointF>();
		}

		protected override void UpdateCreate(PointF current)
		{
			points.Add(current);
			SetBounds(CalcBounds(), false, false);
		}

		protected override void CompleteCreate(PointF end)
		{
			SetBounds(CalcBounds(), false, false);
		}

		protected override bool AllowCreate(PointF current, InteractionState ist)
		{
			return true;
		}

		protected override void UpdateModify(PointF current, InteractionState ist)
		{
			var oldBounds = Bounds;
			base.UpdateModify(current, ist);
            
			var dx = Bounds.X - oldBounds.X;
			var dy = Bounds.Y - oldBounds.Y;

			for (int i = 0; i < points.Count; i++)
			{
				var point = points[i];
				point.X += dx;
				point.Y += dy;
				points[i] = point;
			}
		}

		public override void Draw(IGraphics graphics, RenderOptions options)
		{
			if (points.Count > 1)
			using (var pen = EffectivePen.CreateGdiPen())
				graphics.DrawLines(pen, points.ToArray());
		}

		public override void DrawShadow(IGraphics graphics, RenderOptions options)
		{
		}

		RectangleF CalcBounds()
		{
			float l = float.MaxValue;
			float t = float.MaxValue;
			float r = float.MinValue;
			float b = float.MinValue;

			foreach (var point in points)
			{
				l = Math.Min(l, point.X);
				t = Math.Min(t, point.Y);
				r = Math.Max(r, point.X);
				b = Math.Max(b, point.Y);
			}

			return RectangleF.FromLTRB(l, t, r, b);
		}

		private List<PointF> points;
	}
}